Proximo = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceComm-2.0", "AceHook-2.1")
local L = AceLibrary("AceLocale-2.2"):new("Proximo")
local BZ = AceLibrary("Babble-Zone-2.2")

local prox_list = {}
local msg_header = "Proximo"
local unit = 1
local mainassist = "target"

local Unit_Name = UnitName
local Unit_Class = UnitClass
local Unit_Health = UnitHealth
local Unit_Exists = UnitExists
local Unit_Can_Attack = UnitCanAttack
local Unit_Is_Player = UnitIsPlayer
local Unit_In_Raid = UnitInRaid
local In_Combat_Lockdown = InCombatLockdown
local Get_Num_Party_Members = GetNumPartyMembers
local pairs = pairs

----------------------
--table caching
local new, del
do
	local list = setmetatable({}, {__mode='k'})
	function new(...)
		local t = next(list)
		if t then
			list[t] = nil
			for i = 1, select('#', ...) do
				t[i] = select(i, ...)
			end
			return t
		else
			return { ... }
		end
	end
	function del(t)
		for k in pairs(t) do
			t[k] = nil
		end
		list[t] = true
		return nil
	end
end

----------------------
--On start
function Proximo:OnInitialize()
	self:SetupOptions()
	self:SetCommPrefix(msg_header)
  Proximo:RegisterDB("ProximoDB")
	Proximo:RegisterDefaults("profile", self:GetDefaultConfig())   
	Proximo:RegisterChatCommand({"/proximo", "/prox"}, self.options)
end

----------------------
--Disabled
function Proximo:OnDisable()
	self:ResetList()
	self.frame:Hide()
end

----------------------
--Loaded
function Proximo:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:SetDefaultCommPriority("ALERT")
	self:RegisterComm(msg_header, "BATTLEGROUND", "ReceiveSync")
	self:RegisterComm(msg_header, "PARTY", "ReceiveMainAssist")
end

----------------------
--Player Zoned
function Proximo:PLAYER_ENTERING_WORLD()
	self:ScheduleEvent(self.ZoneCheck, 5, self)
end

----------------------
--Hostile Death
function Proximo:CHAT_MSG_COMBAT_HOSTILE_DEATH(msg)
  local k, v
	for k, v in pairs(prox_list) do
		if msg == string.format(UNITDIESOTHER, k) then
			v.health = 0
			break
		end
  end
end

----------------------
--Zone Check
function Proximo:ZoneCheck()
 	self:RegisterEvent("UNIT_HEALTH")
 	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
 	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT") 
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:ResetList()
	local zone = GetMinimapZoneText()
	if zone == BZ["Ruins of Lordaeron"] or zone == BZ["Nagrand Arena"] or zone == BZ["Blade's Edge Arena"] then
		if self.frame==nil then
			self:MakeFrame()
		end
		self.frame:Show()
		self:ScheduleRepeatingEvent("proximorefresh", self.FrameRefresh, .1, self)
		self:ScheduleRepeatingEvent("proximosync", self.Sync, .5, self)
		self:ScheduleRepeatingEvent("proximoparty", self.PartyTargets, 1, self)
		--enable nameplate tracking if they are on
		if NAMEPLATES_ON then
			self:CreateUnitUpdate()
		end
	else 
	  self:UnregisterEvent("UNIT_HEALTH")
	  self:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	  self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	 	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	  self:CancelScheduledEvent("proximorefresh")
	  self:CancelScheduledEvent("proximosync")
	  self:CancelScheduledEvent("proximoparty")
	  if NAMEPLATES_ON then
			self:StopUnitUpdate()
		end
		if self.frame then self.frame:Hide() end
	end
end

function Proximo:AddToList(name, class, health)
	if not name or name == L["Unknown"] then return end
	local t = new()
	t.class = class
	t.health = health
	t.click = false
	t.unit = unit
  prox_list[name] = t
  --setup frame class
  self:FrameClass(self.frame.buttons[unit],class)
  --update unit count
  unit = unit + 1
  if unit > 5 then unit = 5 end
end

function Proximo:ResetList()
	local k, v
	for k, v in pairs(prox_list) do
		prox_list[k] = del(v)
	end
	unit = 1
	self:FrameClean()
end

function Proximo:GetList()
	return prox_list
end

function Proximo:CheckUnitIDChanges(unitid)
	local name, health=Unit_Name(unitid), Unit_Health(unitid)
	if not prox_list[name] then 
 		if Unit_Can_Attack("player",unitid) and Unit_Is_Player(unitid) and not Unit_In_Raid(unitid) then
 			local lclass, class = Unit_Class(unitid)
			self:AddToList(name, class, health)
		end
	else
		prox_list[name].health = health
	end
end

function Proximo:UPDATE_MOUSEOVER_UNIT()
	self:CheckUnitIDChanges("mouseover")
end
	
function Proximo:PLAYER_TARGET_CHANGED()
 if Unit_Name("target")~=nil then
	 self:CheckUnitIDChanges("target")
 end
end

function Proximo:UNIT_HEALTH()
 if Unit_Name("target")~=nil and arg1=="target" then
	 self:CheckUnitIDChanges("target")
 end
end

function Proximo:PartyTargets()
	local i, party
	for i=1, Get_Num_Party_Members() do
		party = "party"..i.."target"
		if Unit_Exists(party) then
			self:CheckUnitIDChanges(party)
		end
	end
	--update MA, no need to update once combat starts
	if not In_Combat_Lockdown() then
		self:SetMainAssist(self.db.profile.ma)
	end
end

function Proximo:SetMainAssist(ma)
	if Unit_Name("player") == ma then 
		mainassist = "target" 
	else
		local i, party
		for i=1, Get_Num_Party_Members() do
			party = "party"..i
			if Unit_Name(party) == ma then
				mainassist = party.."target" 
				return
			end
		end
	end
end

function Proximo:Sync()
	local k, v
	for k, v in pairs(prox_list) do
		self:SendCommMessage("BATTLEGROUND", k, v.class, v.health)
	end
end

function Proximo:FrameRefresh() 
	local k, v, name, class, health, unit, colortext, alpha
	--update each frame
	for k, v in pairs(prox_list) do
    name, class, health, unit = k, v.class, v.health, v.unit
		alpha, colortext = 0, "|cFFFFFFFF"
		--set colors
		if health <= 0 then
			colortext = "|cFF444444"
		elseif health <= 33 then
			colortext = "|cFFFF0000"
		elseif health <= 66 then
			colortext = "|cFFFFFF00"
		end
    --update status bar
		self.frame.buttons[unit].healthBar:SetValue(health)
		self.frame.buttons[unit].healthtext:SetText(colortext..health.."%|r")
		--update highlight
		if Unit_Name(mainassist) == name then alpha = 1 end
		self.frame.buttons[unit].healthBar.highlight:SetBackdropBorderColor(1,1,0,alpha)
		--update macros if not updated already and not in combat
		if not v.click and not In_Combat_Lockdown() then 
			self.frame.buttons[unit]:SetAttribute("macrotext", "/target "..name)
			v.click = 1
		end
		--Update unit text
    if v.click then 
			self.frame.buttons[unit].text:SetText(colortext..name.."|r")
		else
			self.frame.buttons[unit].text:SetText("** "..colortext..name.." **|r")
    end
  end
end

function Proximo:ReceiveSync(prefix, sender, distribution, name, class, health)
	if not prox_list[name] then
 		self:AddToList(name, class, health)
 	elseif Unit_Name("target") ~= name then
  	prox_list[name].health = health
	end
end

function Proximo:ReceiveMainAssist(prefix, sender, distribution, ma)
	self.db.profile.ma = ma
	self:SetMainAssist(ma)
	self:Print(L["MainAssist"]..": "..ma)
end